import { useEffect } from "react";
import PropTypes from "prop-types";
import { useDispatch, useSelector } from "react-redux";
import { I18n } from "react-polyglot";
import { enGB, es } from "date-fns/locale"
import { LocalizationProvider } from "@mui/x-date-pickers";
import { AdapterDateFns } from "@mui/x-date-pickers/AdapterDateFnsV3";
import navigatorLanguages from "navigator-languages";
import { cambiarIdioma } from "../redux";
import saasMessages from "../i18n";

const supportedLocales = { en: enGB, es };

const IdiomaInner = ({ messages, children }) => {
	const idioma = useSelector(store => store.ui.idioma);

	return (
		<I18n locale={idioma} messages={messages[idioma]}>
			<LocalizationProvider dateAdapter={AdapterDateFns} adapterLocale={supportedLocales[idioma]}>
				{children}
			</LocalizationProvider>
		</I18n>
	);
};

IdiomaInner.propTypes = {
	messages: PropTypes.object,
	children: PropTypes.element.isRequired
};

const getIdiomaNavegador = () => {
	const idiomas = navigatorLanguages();
	if (!idiomas?.length) {
		return undefined;
	}
	const idioma = idiomas[0];
	if (idioma.indexOf("-")) {
		return idioma.substring(0, idioma.indexOf("-"));
	}
	return idioma;
};

const Idioma = ({ messages, idiomaDefecto, children }) => {
	const dispatch = useDispatch();
	const idiomaNavegador = getIdiomaNavegador();

	useEffect(() => {
		let idioma = Reflect.ownKeys(messages)[0];
		if (idiomaNavegador && Object.hasOwn(messages, idiomaNavegador)) {
			idioma = idiomaNavegador;
		}
		else if (idiomaDefecto && Object.hasOwn(messages, idiomaDefecto)) {
			idioma = idiomaDefecto;
		}
		dispatch(cambiarIdioma(idioma));
	}, [dispatch, messages, idiomaDefecto, idiomaNavegador]);

	const m = {};
	Object.keys(messages).forEach(key => {
		m[key] = {
			...messages[key],
			saas: saasMessages[key]
		};
	});

	return (
		<IdiomaInner messages={m}>
			{children}
		</IdiomaInner>
	);
};

Idioma.propTypes = {
	messages: PropTypes.object,
	idiomaDefecto: PropTypes.string,
	children: PropTypes.element.isRequired
};

export default Idioma;
