import {useMemo,useEffect} from "react";
import PropTypes from "prop-types";
import {useDispatch,useSelector} from "react-redux";
import {CssBaseline,useMediaQuery} from "@mui/material";
import {ThemeProvider,createTheme} from "@mui/material/styles";
import {grey} from "@mui/material/colors";
import {cambiarTema} from "../redux";

const Tema=({children}) => {
	const dispatch=useDispatch();
	const {instancia}=useSelector(store => store.aplicacion);
	const tema=useSelector(store => store.ui.tema);
	const prefersDarkMode=useMediaQuery("(prefers-color-scheme: dark)");

	useEffect(() => {
		dispatch(cambiarTema(prefersDarkMode ? "dark" : "light"));
	},[dispatch,prefersDarkMode]);

	const theme=useMemo(() => createTheme({
		palette: {
			mode: tema,
			primary: {
				main: instancia.color.primary
			},
			secondary: {
				main: instancia.color.secondary
			},
			error: {
				main: instancia.color.error
			},
			warning: {
				main: instancia.color.warning
			},
			info: {
				main: instancia.color.info
			},
			success: {
				main: instancia.color.success
			},
			...(tema === "light" && {
				background: {
					default: grey["A200"],
					paper: grey[100]
				}
			})
		},
		shape: {
			borderRadius: 6
		}
	}),[tema,instancia]);

	return (
		<ThemeProvider theme={theme}>
			<CssBaseline />
			{children}
		</ThemeProvider>
	);
};

Tema.propTypes={
	children: PropTypes.element.isRequired
};

export default Tema;