import {createSlice,createAsyncThunk} from "@reduxjs/toolkit";
import {identidadApi} from "../api/inicializarApi";

const colorDefault={
	primary: "#1C6CCC",
	secondary: "#17A7FF",
	error: "#f44336",
	warning: "#ff9800",
	info: "#2196f3",
	success: "#4caf50"
};

const inicializarSlice=createSlice({
	name: "inicializar",
	initialState: {
		inicializando: true,
		inicializado: false,
		instancia: {
			color: colorDefault
		},
		error: null
	},
	extraReducers(builder) {
		builder
			.addCase(inicializar.pending,state => {
				state.inicializando=true;
			})
			.addCase(inicializar.fulfilled,(state,action) => {
				state.inicializando=false;
				state.inicializado=true;
				state.instancia={
					...action.payload,
					abreviatura: action.payload.nombre.match(/\b([A-Z])/g).join(""),
					color: {
						...colorDefault,
						...action.payload.color
					}
				};
				state.error=null;
			})
			.addCase(inicializar.rejected,(state,action) => {
				state.inicializando=false;
				state.inicializado=false;
				state.instancia={
					color: colorDefault
				};
				state.error=action.payload;
			});
	}
});

export const inicializar=createAsyncThunk("inicializar",async (payload) => {
	const response=await identidadApi(payload.devURL,payload.aplicacion);
	return response.data;
});

export const getInstancia=state => state.aplicacion.instancia;

export default inicializarSlice.reducer;