import {createSlice,createAsyncThunk} from "@reduxjs/toolkit";
import {loginApi} from "../api/loginApi";
import {TOKEN,TOKEN_EXPIRATION,REFRESH_TOKEN} from "../Constantes";
import {jwtDecode} from "jwt-decode";

const loginSlice=createSlice({
	name: "login",
	initialState: {
		cargando: false,
		autenticado: false,
		token: null,
		refreshToken: null,
		expiracion: null,
		usuario: null,
		perfiles: [],
		error: null
	},
	reducers: {
		login: (state,action) => {
			const token=action.payload.token;
			const jwtToken=jwtDecode(token);
			const expiracion=new Date();
			expiracion.setSeconds(expiracion.getSeconds() + action.payload.expiracion);
			state.cargando=false;
			state.autenticado=true;
			state.token=token;
			state.refreshToken=action.payload.refreshToken;
			state.expiracion=expiracion.getTime();
			state.usuario=jwtToken.name;
			state.perfiles=jwtToken.authorities;
		},
		logout: state => {
			state.cargando=false;
			state.autenticado=false;
			state.token=null;
			state.refreshToken=null;
			state.expiracion=null;
			state.usuario=null;
			state.perfiles=[];
			state.error=null;
			sessionStorage.removeItem(TOKEN);
			sessionStorage.removeItem(REFRESH_TOKEN);
			sessionStorage.removeItem(TOKEN_EXPIRATION);
		}
	},
	extraReducers(builder) {
		builder
			.addCase(requestToken.pending,state => {
				state.cargando=true;
				state.error=null;
			})
			.addCase(requestToken.fulfilled,(state,action) => {
				const token=action.payload.access_token;
				const jwtToken=jwtDecode(token);
				const expiracion=new Date();
				expiracion.setSeconds(expiracion.getSeconds() + action.payload.expires_in);
				state.cargando=false;
				state.autenticado=true;
				state.token=token;
				state.refreshToken=action.payload.refresh_token;
				state.expiracion=expiracion.getTime();
				state.usuario=jwtToken.name;
				state.perfiles=jwtToken.authorities;
				sessionStorage.setItem(TOKEN,token);
				sessionStorage.setItem(REFRESH_TOKEN,action.payload.refresh_token);
				sessionStorage.setItem(TOKEN_EXPIRATION,expiracion.getTime());
			})
			.addCase(requestToken.rejected,(state,action) => {
				console.log(action);
				state.cargando=false;
				state.autenticado=false;
				state.token=null;
				state.refreshToken=null;
				state.expiracion=null;
				state.usuario=null;
				state.perfiles=[];
				state.error=action.error?.message;
				sessionStorage.removeItem(TOKEN);
				sessionStorage.removeItem(REFRESH_TOKEN);
				sessionStorage.removeItem(TOKEN_EXPIRATION);
			})
			.addCase(refreshToken.pending,state => {
				state.cargando=true;
			})
			.addCase(refreshToken.fulfilled,(state,action) => {
				const expiracion=new Date();
				expiracion.setSeconds(expiracion.getSeconds() + action.payload.expires_in);
				state.token=action.payload.access_token;
				state.refreshToken=action.payload.refresh_token;
				state.expiracion=expiracion.getTime();
				sessionStorage.setItem(TOKEN,action.payload.access_token);
				sessionStorage.setItem(REFRESH_TOKEN,action.payload.refresh_token);
				sessionStorage.setItem(TOKEN_EXPIRATION,expiracion.getTime());
			})
			.addCase(refreshToken.rejected,(state,action) => {
				state.cargando=false;
				state.autenticado=false;
				state.token=null;
				state.refreshToken=null;
				state.expiracion=null;
				state.usuario=null;
				state.perfiles=[];
				state.error=action.error?.message;
				sessionStorage.removeItem(TOKEN);
				sessionStorage.removeItem(REFRESH_TOKEN);
				sessionStorage.removeItem(TOKEN_EXPIRATION);
			});
	}
});

export const requestToken=createAsyncThunk("login/requestToken",async (payload) => {
	const response=await loginApi(payload.devURL).login(payload.clientCredentials,payload.data);
	return response.data;
});

export const refreshToken=createAsyncThunk("login/refreshToken",async (devURL,clientCredentials,refreshToken) => {
	const response=await loginApi(devURL).refreshToken(clientCredentials,refreshToken);
	return response.data;
});

export const {login,logout}=loginSlice.actions;

export const getToken=state => state.login.token;

export const getUsuario=state => state.login.usuario;

export const getPerfiles=state => state.login.perfiles;

export default loginSlice.reducer;
