import lodashMemoize from "lodash/memoize";

const EMAIL_REGEX=/^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;

const isEmpty=(value) =>
    typeof value === 'undefined' ||
    value === null ||
    value === '' ||
    (Array.isArray(value) && value.length === 0);

const getMessage=(message,messageArgs,value,values) => (
	typeof message === 'function'
	? message({args: messageArgs,value,values})
	: messageArgs
	? {message,args: messageArgs}
	: message
);

const memoize=(fn) => lodashMemoize(fn,(...args) => JSON.stringify(args));

export const composeValidators=(...validators) => (value,values,meta) => {
	const allValidators=Array.isArray(validators[0]) ? validators[0] : validators;
    return allValidators.reduce(
        (error,validator) =>
            error ||
            (typeof validator === 'function' && validator(value,values,meta)),
        undefined
    );
};

export const required=memoize((message="validacion.obligatorio") => (
	Object.assign(
		(value,values) => isEmpty(value) ? getMessage(message,undefined,value,values) : undefined,
		{isRequired: true}
	)
));

export const requiredNotFalse=memoize((message="validacion.obligatorio") => (
	Object.assign(
		(value,values) => isEmpty(value) || !value ? getMessage(message,undefined,value,values) : undefined,
		{isRequired: true}
	)
));

export const number=memoize((message="validacion.numero") => (value,values) => (
    !isEmpty(value) && isNaN(value) ? getMessage(message,undefined,value,values) : undefined
));

export const minLength=memoize((min,message="validacion.longitud.minima") => (value,values) =>
	!isEmpty(value) && value.length < min ? getMessage(message,undefined,value,values) : undefined
);

export const maxLength=memoize((max,message="validacion.longitud.maxima") => (value,values) =>
	!isEmpty(value) && value.length > max ? getMessage(message,undefined,value,values) : undefined
);

export const min=memoize((min,message="validacion.minimo") => (value,values) =>
	!isEmpty(value) && parseFloat(value) < min ? getMessage(message,undefined,value,values) : undefined
);

export const max=memoize((max,message="validacion.maximo") => (value,values) =>
	!isEmpty(value) && parseFloat(value) > max ? getMessage(message,undefined,value,values) : undefined
);

export const regex=lodashMemoize((pattern,message="validacion.regex") => (value,values) =>
	!isEmpty(value) && typeof value === "string" && !pattern.test(value) ? getMessage(message,undefined,value,values) : undefined,
    (pattern,message) => (pattern.toString() + message)
);

export const email=memoize((message="validacion.correo") => regex(EMAIL_REGEX,message));

export const remote=memoize((call,params,message="validacion.remote") => (value,values,meta) => {
	return meta.active && !isEmpty(value) &&
	call(value,params)
	.then(data => data ? getMessage(message,undefined,value,values) : undefined)
	.catch(error => getMessage(error.message,undefined,value,values))
});
